//
// 5th order Runge Kutta step with quality-controlled step size,
// patterned after Num Recipes page 574, 1st ed.  Takes u(t) and du/dt(t),
// where du/dt is created by a call to derivs() prior to calling
// this routine.  u(t) is replaced by u(t+h) on output, where h is
// hdid, and hnext is the estimated next step size.  EPS here is
// the fractional tolerance desired.  first written 2/16/93, and updated
// to C++ 2/21/06.
//

#include "lorenz.h"

void rkqc(int n, dbl htry, dbl t, vector<dbl>& u,
            vector<dbl>& dudt, dbl& hdid, dbl& hnext,
            void derivs( dbl, vector<dbl>&, vector<dbl>&))
{
    int i ;
    dbl h, hh, temp, errmax;

    vector<dbl> usav (n);  // for storing u (n: no of dependent variables)
    vector<dbl> dusav (n); // for storing dudt
    vector<dbl> ut (n);    // temp u
    vector<dbl> dut (n);   // temp dudt
    vector<dbl> uscal (n); // scale factors

    usav = u;     // save u
    dusav = dudt; // save dudt

    h = htry ;
    int count = 0;
    for ( ; ; ){
        count++;
//        cout << "count = " << count << '\n';

// TAKE 2 HALF STEPS
        hh = 0.5 * h;
        rk(n, hh, t, u, dudt, derivs);
        derivs(t, u, dudt);
        rk(n, hh, t+hh, u, dudt, derivs);
                                      // u here is used on success

// AND 1 WHOLE STEP
        ut = usav;   // retrieve original u
        dut = dusav; // retrieve original dudt
        rk(n, h, t, ut, dut, derivs);

// AND COMPARE TO EVALUATE ACCURACY...

  // compute scale factors
        for (i = 0 ; i < n ; i++){
            uscal[i] = fabs(u[i]) + h * fabs(dudt[i]) + TINY;
        }

  // compute maximum error
        errmax = 0.0;
        for (i = 0 ; i < n ; i++){
            ut[i] = u[i] - ut[i];             // ut has error now
            temp = fabs( ut[i] / uscal[i] );  // fractional error
            if (errmax < temp) errmax = temp;
        }
        errmax /= EPS;          // EPS is the desired tolerance

  // if error is small enough, save h and compute next step size
        if (errmax <= 1.0){
            hdid = h;
            hnext = (errmax > ERRCON ?
               SAFETY * h * exp(PGROW * log(errmax)) :
               4.0 * h);
            break;              // get out of "for" loop
        }

  // else reduce h and start over
        h = SAFETY * h * exp(PSHRINK * log(errmax));
        u = usav;               // restore saved values
        dudt = dusav;
    }  // end of "for" loop

// AT THE END CORRECT FOR FIFTH ORDER ESTIMATE OF ERROR

    for (i = 0 ; i < n ; i++){
        u[i] += ut[i] * FCOR;
    }
}

//    free( usav ) ;
//    free( dusav ) ;
//    free( ut ) ;
//    free( dut ) ;
//    free( uscal ) ;

